# Peanutbutter

A lock screen for linux mobile. Specially designed for Sxmo (sway edition).

## Usage

Compile this repo via `cargo build` and move `peanutbutter` into your `$PATH`.
Add `peanutbutter` to your `sxmo_hook_screenoff.sh` hook.

For example, my `sxmo_hook_screenoff.sh` looks like:

``` sh
#!/bin/sh
# configversion: 63228ca431ea6d18709d2b3ad80aabf6
# SPDX-License-Identifier: AGPL-3.0-only
# Copyright 2022 Sxmo Contributors
#
# See man 7 sxmo.states.
#
# This will:
# - turn screen off
# - turn input off
# - launch a demon to blink purple led every 2s
# - check wakelocks and if none suspend after 3s hold

# include common definitions
# shellcheck source=scripts/core/sxmo_common.sh
. sxmo_common.sh

# This hook is called when the system reaches a off state (screen off)

sxmo_led.sh blink red blue &

[ "$SXMO_WM" = "sway" ] && swaymsg mode default
sxmo_wm.sh dpms on
sxmo_wm.sh inputevent touchscreen off

sxmo_jobs.sh start periodic_blink sxmo_run_periodically.sh - 2 sxmo_led.sh blink red blue

case "$SXMO_WM" in
	dwm)
		# dmenu will grab input focus (i.e. power button) so kill it before going to
		# screenoff unless proximity lock is running (i.e. there's a phone call).
		if ! sxmo_jobs.sh running proximity_lock -q; then
			sxmo_dmenu.sh close
		fi
		;;
esac

wait

peanutbutter &
```

## Debugging

If peanutbutter crashes, it will display a red screen.
To debug, run the following command in your terminal (on the phone):

`peanutbutter 2> /tmp/peanutbutter.log`

When peanutbutter crashes, ssh into your phone and check the `/tmp/peanutbutter.log` file for the error message.

# Controls

The screen is segregated into 4 corners. The passcode consists of 4 touches.
If you make a mistake while entering the passcode, tap the bottom right of the screen.
The default passcode is: `TOP_LEFT`, `TOP_LEFT`, `BOTTOM_LEFT`, `TOP_RIGHT`.
You can change the default passcode in the `touch_pass.rs` file and recompile peanutbutter.


# Safety

This program uses: `ext-session-lock-v1`.
If `peanutbutter` crashes, your information is not exposed.
Try `pkill peanutbutter`.

# TODO

- Clean up code (compiler warnings)
- Package
- Show an incoming call and pickup easily via lock screen
- Automatically detect screen width and height
