use std::fmt;


const TOUCHES: u32 = 4;
const PASSCODE: [PasscodeItem; TOUCHES as usize] = [
    PasscodeItem::TopLeft,
    PasscodeItem::TopLeft,
    PasscodeItem::BottomLeft,
    PasscodeItem::TopRight,
];

#[derive(Copy, Clone, PartialEq)]
enum PasscodeItem {
    TopLeft,
    TopRight,
    BottomLeft,
    Unknown,
}

impl fmt::Display for PasscodeItem {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self {
            PasscodeItem::TopLeft => write!(f, "TOP LEFT"),
            PasscodeItem::TopRight => write!(f, "TOP RIGHT"),
            PasscodeItem::BottomLeft => write!(f, "BOTTOM LEFT"),
            PasscodeItem::Unknown => write!(f, "UNKNOWN"),
        }
    }
}

pub struct TouchState {
    last_touch: u32,
    touches: [PasscodeItem; TOUCHES as usize],
    half_width: f64,
    half_height: f64,
}

impl TouchState {
    pub fn new(width: i32, height: i32) -> Self {
        Self {
            last_touch: 0,
            touches: [PasscodeItem::Unknown; (TOUCHES as usize)],
            half_width: width as f64 / 2.0,
            half_height: height as f64 / 2.0,
        }
    }

    pub fn add_touch(&mut self, x_val: f64, y_val: f64) {
        let mut touch = PasscodeItem::Unknown;

        if x_val > self.half_width && y_val > self.half_height {
            self.clear();
            return;
        }

        if x_val < self.half_width && y_val < self.half_height {
            touch = PasscodeItem::TopLeft;
        } else if x_val > self.half_width && y_val < self.half_height {
            touch = PasscodeItem::TopRight;
        } else if x_val < self.half_width && y_val > self.half_height {
            touch = PasscodeItem::BottomLeft;
        }

        self.touches[self.last_touch as usize] = touch;

        self.last_touch += 1;
        if self.last_touch >= TOUCHES {
            self.last_touch = 0;
        }
    }

    pub fn print(&self) {
        for touch in self.touches.iter() {
            println!("{}", touch);
        }
    }

    fn clear(&mut self) {
        self.touches = [PasscodeItem::Unknown; (TOUCHES as usize)];
        self.last_touch = 0;
    }

    pub fn is_pass_valid(&self) -> bool {
        self.touches == PASSCODE
    }
}
